#pragma option t        24-bit integer to 32-bit floating point conversion


void flo2432(void)
{
#asm
;       PIC16 32 BIT FLOATING POINT LIBRARY     VERSION 2.8  1996/10/07
;
;       Binary operations: input in AEXP,AARG and BEXP,BARG with output in AEXP,AARG
;
;       All routines return WREG = 0x00 for successful completion, and WREG = 0xFF
;       for an error condition specified in FPFLAGS.
;
;*****************************************************************************************
;*****************************************************************************************
;
;       Integer to float conversion
;
;       Input:  24 bit 2's complement integer right justified in AARGB0, AARGB1, AARGB2
;
;       Output: 32 bit floating point number in AEXP, AARGB0, AARGB1, AARGB2
;
;       Result: AARG  <--  FLOAT( AARG )
;
;       PM: 69                               DM: 7
;
;*****************************************************************************************
;
FLO2432
FLO32         MOVLW     0x17+EXPBIAS   ;initialize exponent and add bias
              MOVWF     EXP
              CLRF      SIGN
              BTFSS     AARGB0,MSB     ;test sign
              GOTO      NRM3232F
              COMF      AARGB2,F       ;if < 0, negate and set MSB in SIGN
              COMF      AARGB1,F
              COMF      AARGB0,F
              INCF      AARGB2,F
              BTFSC     _Z
              INCF      AARGB1,F
              BTFSC     _Z
              INCF      AARGB0,F
              BSF       SIGN,MSB

;
NRM3232F
NRM32         CLRF      TEMP           ;clear exponent decrement
              MOVF      AARGB0,W       ;test if highbyte=0
              BTFSS     _Z
              GOTO      NORM3232F
              MOVF      AARGB1,W       ;if so, shift 8 bits by move
              MOVWF     AARGB0
              MOVF      AARGB2,W
              MOVWF     AARGB1
              CLRF      AARGB2
              BSF       TEMP,3         ;increase decrement by 8

              MOVF      AARGB0,W       ;test if highbyte=0
              BTFSS     _Z
              GOTO      NORM3232F
              MOVF      AARGB1,W       ;if so, shift 8 bits by move
              MOVWF     AARGB0
              CLRF      AARGB1
              BCF       TEMP,3         ;increase decrement by 8
              BSF       TEMP,4
        
              MOVF      AARGB0,W       ;if highbyte=0, result=0
              BTFSC     _Z
              GOTO      RES032F

NORM3232F     MOVF      TEMP,W
              SUBWF     EXP,F
              BTFSS     _Z
              BTFSS     _C
              GOTO      SETFUN32F
              BCF       _C             ;clear carry bit

NORM3232AF    BTFSC     AARGB0,MSB     ;if MSB=1, normalization done
              GOTO      FIXSIGN32F
              RLF       AARGB2,F       ;otherwise, shift left and 
              RLF       AARGB1,F       ;decrement EXP
              RLF       AARGB0,F
              DECFSZ    EXP,F
              GOTO      NORM3232AF
              GOTO      SETFUN32F      ;underflow if EXP=0

FIXSIGN32F    BTFSS     SIGN,MSB
              BCF       AARGB0,MSB     ;clear explicit MSB if positive
              RETLW     0x00

RES032F       CLRF      AARGB0         ;result equals zero
              CLRF      AARGB1 
              CLRF      AARGB2
              CLRF      AARGB3
              CLRF      EXP
              RETLW     0x00

SETFUN32F     BSF       FPFLAGS,FUN    ;set floating point underflag
              BTFSS     FPFLAGS,SAT    ;test for saturation
              RETLW     0xFF           ;return error code in WREG

              MOVLW     0x01           ;saturate to smallest floating
              MOVWF     AEXP           ;point number = 0x 01 00 00 00
              CLRF      AARGB0         ;modulo the appropriate sign bit
              CLRF      AARGB1
              CLRF      AARGB2
              RLF       SIGN, F
              RRF       AARGB0, F
              RETLW     0xFF           ;return error code in WREG

#endasm
}
