#pragma option t        32-bit integer to 32-bit floating point conversion


void flo3232(void)
{
#asm
;       PIC16 32 BIT FLOATING POINT LIBRARY     VERSION 2.8  1996/10/07
;
;       Binary operations: input in AEXP,AARG and BEXP,BARG with output in AEXP,AARG
;
;       All routines return WREG = 0x00 for successful completion, and WREG = 0xFF
;       for an error condition specified in FPFLAGS.
;
;*****************************************************************************************
;*****************************************************************************************
;
;       Integer to float conversion
;
;       Input:  32 bit 2's complement integer right justified in AARGB0, AARGB1, AARGB2,
;               AARGB3
;
;       Output: 32 bit floating point number in AEXP, AARGB0, AARGB1, AARGB2
;
;       Result: AARG  <--  FLOAT( AARG )
;
;       PM: 114                          DM: 8
;*****************************************************************************************
;
FLO3232       MOVLW     0x1F+EXPBIAS   ;initialize exponent and add bias
              MOVWF     EXP
              CLRF      SIGN
              BTFSS     AARGB0,MSB     ;test sign
              GOTO      NRM4032I
              COMF      AARGB3,F       ;if < 0, negate and set MSB in SIGN
              COMF      AARGB2,F
              COMF      AARGB1,F
              COMF      AARGB0,F
              INCF      AARGB3,F
              BTFSC     _Z
              INCF      AARGB2,F
              BTFSC     _Z
              INCF      AARGB1,F
              BTFSC     _Z
              INCF      AARGB0,F
              BSF       SIGN,MSB

NRM4032I      CLRF      TEMP           ;clear exponent decrement
              MOVF      AARGB0,W       ;test if highbyte=0
              BTFSS     _Z
              GOTO      NORM4032I
              MOVF      AARGB1,W       ;if so, shift 8 bits by move
              MOVWF     AARGB0
              MOVF      AARGB2,W
              MOVWF     AARGB1
              MOVF      AARGB3,W
              MOVWF     AARGB2
              CLRF      AARGB3
              BSF       TEMP,3         ;increase decrement by 8

              MOVF      AARGB0,W       ;test if highbyte=0
              BTFSS     _Z
              GOTO      NORM4032I
              MOVF      AARGB1,W       ;if so, shift 8 bits by move
              MOVWF     AARGB0
              MOVF      AARGB2,W
              MOVWF     AARGB1
              CLRF      AARGB2
              BCF       TEMP,3         ;increase decrement by 8
              BSF       TEMP,4
        
              MOVF      AARGB0,W       ;test if highbyte=0
              BTFSS     _Z
              GOTO      NORM4032I
              MOVF      AARGB1,W       ;if so, shift 8 bits by move
              MOVWF     AARGB0
              CLRF      AARGB1
              BSF       TEMP,3         ;increase decrement by 8
        
              MOVF      AARGB0,W       ;if highbyte=0, result=0
              BTFSC     _Z
              GOTO      RES032I

NORM4032I     MOVF      TEMP,W
              SUBWF     EXP,F
              BTFSS     _Z
              BTFSS     _C
              GOTO      SETFUN32I
              BCF       _C             ;clear carry bit

NORM4032I1    BTFSC     AARGB0,MSB     ;if MSB=1, normalization done
              GOTO      NRMRND4032I
              RLF       AARGB3,F       ;otherwise, shift left and 
              RLF       AARGB2,F       ;decrement EXP
              RLF       AARGB1,F
              RLF       AARGB0,F
              DECFSZ    EXP, F
              GOTO      NORM4032I1
              GOTO      SETFUN32I      ;underflow if EXP=0

NRMRND4032I   BTFSC     FPFLAGS,RND
              BTFSS     AARGB2,LSB
              GOTO      FIXSIGN32I
              BTFSS     AARGB3,MSB     ;round if next bit is set
              GOTO      FIXSIGN32I
              INCF      AARGB2,F
              BTFSC     _Z
              INCF      AARGB1,F
              BTFSC     _Z
              INCF      AARGB0,F

              BTFSS     _Z             ;has rounding caused carryout?
              GOTO      FIXSIGN32I
              RRF       AARGB0,F       ;if so, right shift
              RRF       AARGB1,F
              RRF       AARGB2,F
              INCF      EXP,F
              BTFSC     _Z             ;check for overflow
              GOTO      SETFOV32I
              GOTO      FIXSIGN32I

SETFOV32I     BSF       FPFLAGS,FOV    ;set floating point underflag
              BTFSS     FPFLAGS,SAT    ;test for saturation
              RETLW     0xFF           ;return error code in WREG

              MOVLW     0xFF
              MOVWF     AEXP           ;saturate to largest floating
              MOVWF     AARGB0         ;point number = 0x FF 7F FF FF
              MOVWF     AARGB1         ;modulo the appropriate sign bit
              MOVWF     AARGB2
              RLF       SIGN, F
              RRF       AARGB0, F
              RETLW     0xFF           ;return error code in WREG

SETFUN32I     BSF       FPFLAGS,FUN    ;set floating point underflag
              BTFSS     FPFLAGS,SAT    ;test for saturation
              RETLW     0xFF           ;return error code in WREG

              MOVLW     0x01           ;saturate to smallest floating
              MOVWF     AEXP           ;point number = 0x 01 00 00 00
              CLRF      AARGB0         ;modulo the appropriate sign bit
              CLRF      AARGB1
              CLRF      AARGB2
              RLF       SIGN, F
              RRF       AARGB0, F
              RETLW     0xFF           ;return error code in WREG


FIXSIGN32I    BTFSS     SIGN,MSB
              BCF       AARGB0,MSB     ;clear explicit MSB if positive
              RETLW     0x00

RES032I       CLRF      AARGB0         ;result equals zero
              CLRF      AARGB1 
              CLRF      AARGB2
              CLRF      AARGB3
              CLRF      EXP
              RETLW     0x00

#endasm
}
