#pragma option t        32-bit floating point divide


void fpd32(void)
{
#asm
;       PIC16 32 BIT FLOATING POINT LIBRARY     VERSION 2.8  1996/10/07
;
;       Binary operations: input in AEXP,AARG and BEXP,BARG with output in AEXP,AARG
;
;       All routines return WREG = 0x00 for successful completion, and WREG = 0xFF
;       for an error condition specified in FPFLAGS.
;
;*****************************************************************************************
;*****************************************************************************************
;
;       Floating Point Divide
;
;       Input:  32 bit floating point dividend in AEXP, AARGB0, AARGB1, AARGB2
;               32 bit floating point divisor in BEXP, BARGB0, BARGB1, BARGB2
;
;       Output: 32 bit floating point quotient in AEXP, AARGB0, AARGB1, AARGB2
;
;       Result: AARG  <--  AARG / BARG
;
;       PM: 166                                                DM: 14
;
;*****************************************************************************************
;
FPD32         MOVF      BEXP,W         ;test for divide by zero
              BTFSC     _Z
              GOTO      SETFDZ32
              MOVF      AEXP,W
              BTFSC     _Z
              GOTO      RES032D

D32BNE0       MOVF      AARGB0,W
              XORWF     BARGB0,W
              MOVWF     SIGN           ;save sign in SIGN
              BSF       AARGB0,MSB     ;make argument MSB's explicit
              BSF       BARGB0,MSB

TALIGN32      CLRF      TEMP           ;clear align increment
              MOVF      AARGB0,W
              MOVWF     AARGB3         ;test for alignment
              MOVF      AARGB1,W
              MOVWF     AARGB4
              MOVF      AARGB2,W
              MOVWF     AARGB5

              MOVF      BARGB2,W
              SUBWF     AARGB5,F
              MOVF      BARGB1,W
              BTFSS     _C
              INCFSZ    BARGB1,W

TS1ALIGN32    SUBWF     AARGB4,F
              MOVF      BARGB0,W
              BTFSS     _C
              INCFSZ    BARGB0,W

TS2ALIGN32    SUBWF     AARGB3,F
              CLRF      AARGB3
              CLRF      AARGB4
              CLRF      AARGB5
              BTFSS     _C
              GOTO      DALIGN32OK

              BCF       _C             ;align if necessary
              RRF       AARGB0,F
              RRF       AARGB1,F
              RRF       AARGB2,F
              RRF       AARGB3,F
              MOVLW     0x01
              MOVWF     TEMP           ;save align increment          

DALIGN32OK    MOVF      BEXP,W         ;compare AEXP and BEXP
              SUBWF     EXP,F
              BTFSS     _C
              GOTO      ALTB32
        
AGEB32        MOVLW     EXPBIAS-1
              ADDWF     TEMP,W
              ADDWF     EXP,F
              BTFSC     _C
              GOTO      SETFOV32D
              GOTO      DARGOK32       ;set overflow flag

ALTB32        MOVLW     EXPBIAS-1
              ADDWF     TEMP,W
              ADDWF     EXP,F
              BTFSS     _C
              GOTO      SETFUN32D      ;set underflow flag

DARGOK32      MOVLW     0x18           ;initialize counter
              MOVWF     TEMPB1

DLOOP32       RLF       AARGB5,F       ;left shift
              RLF       AARGB4,F
              RLF       AARGB3,F
              RLF       AARGB2,F
              RLF       AARGB1,F
              RLF       AARGB0,F
              RLF       TEMP,F

              MOVF      BARGB2,W       ;subtract
              SUBWF     AARGB2,F
              MOVF      BARGB1,W
              BTFSS     _C
              INCFSZ    BARGB1,W
DS132         SUBWF     AARGB1,F

              MOVF      BARGB0,W
              BTFSS     _C
              INCFSZ    BARGB0,W
DS232         SUBWF     AARGB0,F

              RLF       BARGB0,W
              IORWF     TEMP,F
              BTFSS     TEMP,LSB       ;test for restore
              GOTO      DREST32
              BSF       AARGB5,LSB
              GOTO      DOK32

DREST32       MOVF      BARGB2,W       ;restore if necessary
              ADDWF     AARGB2,F
              MOVF      BARGB1,W
              BTFSC     _C
              INCFSZ    BARGB1,W
DAREST32      ADDWF     AARGB1,F
              MOVF      BARGB0,W
              BTFSC     _C
              INCF      BARGB0,W
              ADDWF     AARGB0,F
              BCF       AARGB5,LSB

DOK32         DECFSZ    TEMPB1,F
              GOTO      DLOOP32

DROUND32      BTFSC     FPFLAGS,RND
              BTFSS     AARGB5,LSB
              GOTO      DIV32OK
              BCF       _C
              RLF       AARGB2,F       ;compute next significant bit
              RLF       AARGB1,F       ;for rounding
              RLF       AARGB0,F
              RLF       TEMP,F

              MOVF      BARGB2,W       ;subtract
              SUBWF     AARGB2,F
              MOVF      BARGB1,W
              BTFSS     _C
              INCFSZ    BARGB1,W
              SUBWF     AARGB1,F

              MOVF      BARGB0,W
              BTFSS     _C
              INCFSZ    BARGB0,W
              SUBWF     AARGB0,F

              RLF       BARGB0,W
              IORWF     TEMP,W
              ANDLW     0x01            
              ADDWF     AARGB5,F
              BTFSC     _C
              INCF      AARGB4,F
              BTFSC     _Z
              INCF      AARGB3,F
              BTFSS     _Z             ;test if rounding caused carryout
              GOTO      DIV32OK
              RRF       AARGB3,F
              RRF       AARGB4,F
              RRF       AARGB5,F
              INCF      EXP,F
              BTFSC     _Z             ;test for overflow
              GOTO      SETFOV32D

DIV32OK       BTFSS     SIGN,MSB
              BCF       AARGB3,MSB     ;clear explicit MSB if positive

              MOVF      AARGB3,W
              MOVWF     AARGB0         ;move result to AARG
              MOVF      AARGB4,W
              MOVWF     AARGB1
              MOVF      AARGB5,W
              MOVWF     AARGB2
              RETLW     0x00

SETFUN32D     BSF       FPFLAGS,FUN    ;set floating point underflag
              BTFSS     FPFLAGS,SAT    ;test for saturation
              RETLW     0xFF           ;return error code in WREG

              MOVLW     0x01           ;saturate to smallest floating
              MOVWF     AEXP           ;point number = 0x 01 00 00 00
              CLRF      AARGB0         ;modulo the appropriate sign bit
              CLRF      AARGB1
              CLRF      AARGB2
              RLF       SIGN,F
              RRF       AARGB0,F
              RETLW     0xFF           ;return error code in WREG

SETFDZ32      BSF       FPFLAGS,FDZ    ;set divide by zero flag
              RETLW     0xFF

RES032D       CLRF      AARGB0         ;result equals zero
              CLRF      AARGB1         ;clear variables
              CLRF      AARGB2
              CLRF      AARGB3
              CLRF      EXP
              RETLW     0x00

SETFOV32D     BSF       FPFLAGS,FOV    ;set floating point underflag
              BTFSS     FPFLAGS,SAT    ;test for saturation
              RETLW     0xFF           ;return error code in WREG

              MOVLW     0xFF
              MOVWF     AEXP           ;saturate to largest floating
              MOVWF     AARGB0         ;point number = 0x FF 7F FF FF
              MOVWF     AARGB1         ;modulo the appropriate sign bit
              MOVWF     AARGB2
              RLF       SIGN,F
              RRF       AARGB0,F
              RETLW     0xFF           ;return error code in WREG

#endasm
}
