#pragma option t        32-bit floating point multiply


void fpm32(void)
{
#asm
;       PIC16 32 BIT FLOATING POINT LIBRARY     VERSION 2.8  1996/10/07
;
;       Binary operations: input in AEXP,AARG and BEXP,BARG with output in AEXP,AARG
;
;       All routines return WREG = 0x00 for successful completion, and WREG = 0xFF
;       for an error condition specified in FPFLAGS.
;
;*****************************************************************************************
;*****************************************************************************************
;
;       Floating Point Multiply
;
;       Input:  32 bit floating point number in AEXP, AARGB0, AARGB1, AARGB2
;               32 bit floating point number in BEXP, BARGB0, BARGB1, BARGB2
;
;       Output: 32 bit floating point product in AEXP, AARGB0, AARGB1, AARGB2
;
;       Result: AARG  <--  AARG * BARG
;
;       PM: 111                                   DM: 14
;
;*****************************************************************************************
;
FPM32         MOVF      AEXP,W         ;test for zero arguments
              BTFSS     _Z
              MOVF      BEXP,W
              BTFSC     _Z
              GOTO      RES032M

M32BNE0       MOVF      AARGB0,W
              XORWF     BARGB0,W
              MOVWF     SIGN           ;save sign in SIGN
              MOVF      BEXP,W
              ADDWF     EXP, F
              MOVLW     EXPBIAS-1
              BTFSS     _C
              GOTO      MTUN32

              SUBWF     EXP,F
              BTFSC     _C
              GOTO      SETFOV32M      ;set multiply overflow flag
              GOTO      MOK32

MTUN32        SUBWF     EXP,F
              BTFSS     _C
              GOTO      SETFUN32M

MOK32         MOVF      AARGB0,W
              MOVWF     AARGB3
              MOVF      AARGB1,W
              MOVWF     AARGB4
              MOVF      AARGB2,W
              MOVWF     AARGB5
              BSF       AARGB3,MSB            
              BSF       BARGB0,MSB
              BCF       _C
              CLRF      AARGB0         ;clear initial partial product
              CLRF      AARGB1
              CLRF      AARGB2
              MOVLW     0x18
              MOVWF     TEMP           ;initialize counter

MLOOP32       BTFSS     AARGB5,LSB     ;test high byte
              GOTO      MNOADD32

MADD32        MOVF      BARGB2,W
              ADDWF     AARGB2,F
              MOVF      BARGB1,W
              BTFSC     _C
              INCFSZ    BARGB1,W
              ADDWF     AARGB1,F
              MOVF      BARGB0,W
              BTFSC     _C
              INCFSZ    BARGB0,W
              ADDWF     AARGB0,F

MNOADD32      RRF       AARGB0,F
              RRF       AARGB1,F
              RRF       AARGB2,F
              RRF       AARGB3,F
              RRF       AARGB4,F
              RRF       AARGB5,F
              BCF       _C
              DECFSZ    TEMP,F
              GOTO      MLOOP32

              BTFSC     AARGB0,MSB     ;check for postnormalization
              GOTO      MROUND32
              RLF       AARGB3,F
              RLF       AARGB2,F
              RLF       AARGB1,F
              RLF       AARGB0,F
              DECF      EXP,F

MROUND32      BTFSC     FPFLAGS,RND
              BTFSS     AARGB2,LSB
              GOTO      MUL32OK
              BTFSS     AARGB3,MSB
              GOTO      MUL32OK
              INCF      AARGB2,F
              BTFSC     _Z
              INCF      AARGB1,F
              BTFSC     _Z
              INCF      AARGB0,F

              BTFSS     _Z             ;has rounding caused carryout?
              GOTO      MUL32OK
              RRF       AARGB0,F       ;if so, right shift
              RRF       AARGB1,F
              RRF       AARGB2,F
              INCF      EXP,F
              BTFSC     _Z             ;check for overflow
              GOTO      SETFOV32M

MUL32OK       BTFSS     SIGN,MSB
              BCF       AARGB0,MSB     ;clear explicit MSB if positive
              RETLW     0x00

SETFOV32M     BSF       FPFLAGS,FOV    ;set floating point underflag
              BTFSS     FPFLAGS,SAT    ;test for saturation
              RETLW     0xFF           ;return error code in WREG

              MOVLW     0xFF
              MOVWF     AEXP           ;saturate to largest floating
              MOVWF     AARGB0         ;point number = 0x FF 7F FF FF
              MOVWF     AARGB1         ;modulo the appropriate sign bit
              MOVWF     AARGB2
              RLF       SIGN,F
              RRF       AARGB0,F
              RETLW     0xFF           ;return error code in WREG

RES032M       CLRF      AARGB0         ;result equals zero
              CLRF      AARGB1         ;clear variables
              CLRF      AARGB2
              CLRF      AARGB3
              CLRF      EXP
              RETLW     0x00

SETFUN32M     BSF       FPFLAGS,FUN    ;set floating point underflag
              BTFSS     FPFLAGS,SAT    ;test for saturation
              RETLW     0xFF           ;return error code in WREG

              MOVLW     0x01           ;saturate to smallest floating
              MOVWF     AEXP           ;point number = 0x 01 00 00 00
              CLRF      AARGB0         ;modulo the appropriate sign bit
              CLRF      AARGB1
              CLRF      AARGB2
              RLF       SIGN,F
              RRF       AARGB0,F
              RETLW     0xFF           ;return error code in WREG

#endasm
}
