#pragma option t        32-bit floating point subtraction and addition


void fps32(void)
{
#asm
;       PIC16 32 BIT FLOATING POINT LIBRARY     VERSION 2.8  1996/10/07
;
;       Binary operations: input in AEXP,AARG and BEXP,BARG with output in AEXP,AARG
;
;       All routines return WREG = 0x00 for successful completion, and WREG = 0xFF
;       for an error condition specified in FPFLAGS.
;
;*****************************************************************************************
;
;       Floating Point Subtract
;
;       Input:  32 bit floating point number in AEXP, AARGB0, AARGB1, AARGB2
;               32 bit floating point number in BEXP, BARGB0, BARGB1, BARGB2
;
;       Output: 32 bit floating point sum in AEXP, AARGB0, AARGB1, AARGB2
;
;       Result: AARG  <--  AARG - BARG
;
;       PM: 3+270 = 273                         DM: 14
;
;*****************************************************************************************
;
FPS32         MOVLW     0x80
              XORWF     BARGB0,F
              GOTO      FPA32
#endasm
}


void fpa32(void)
{
#asm

;
;       Floating Point Addition
;
;       Input:  32 bit floating point number in AEXP, AARGB0, AARGB1, AARGB2
;               32 bit floating point number in BEXP, BARGB0, BARGB1, BARGB2
;
;       Output: 32 bit floating point sum in AEXP, AARGB0, AARGB1, AARGB2
;
;       Result: AARG  <--  AARG - BARG
;
;       PM: 270                        DM: 14
;
;*****************************************************************************************
;
FPA32         MOVF      AARGB0,W       ;exclusive or of signs in TEMP
              XORWF     BARGB0,W
              MOVWF     TEMP
              CLRF      AARGB3         ;clear extended byte
              CLRF      BARGB3

              MOVF      AEXP,W         ;use AARG if AEXP >= BEXP
              SUBWF     BEXP,W
              BTFSS     _C
              GOTO      USEA32S

              MOVF      BEXP,W         ;use BARG if AEXP < BEXP
              MOVWF     AARGB5         ;therefore, swap AARG and BARG
              MOVF      AEXP,W
              MOVWF     BEXP
              MOVF      AARGB5,W
              MOVWF     AEXP

              MOVF      BARGB0,W
              MOVWF     AARGB5
              MOVF      AARGB0,W
              MOVWF     BARGB0
              MOVF      AARGB5,W
              MOVWF     AARGB0

              MOVF      BARGB1,W
              MOVWF     AARGB5
              MOVF      AARGB1,W
              MOVWF     BARGB1
              MOVF      AARGB5,W
              MOVWF     AARGB1

              MOVF      BARGB2,W
              MOVWF     AARGB5
              MOVF      AARGB2,W
              MOVWF     BARGB2
              MOVF      AARGB5,W
              MOVWF     AARGB2

USEA32S       MOVF      BEXP,W         ;return AARG if BARG = 0
              BTFSC     _Z
              RETLW     0x00

              MOVF      AARGB0,W
              MOVWF     SIGN           ;save sign in SIGN
              BSF       AARGB0,MSB     ;make MSB's explicit
              BSF       BARGB0,MSB

              MOVF      BEXP,W         ;compute shift count in BEXP
              SUBWF     AEXP,W
              MOVWF     BEXP
              BTFSC     _Z
              GOTO      ALIGNED32S

              MOVLW     8
              SUBWF     BEXP,W
              BTFSS     _C             ;if BEXP >= 8, do byte shift
              GOTO      ALIGNB32S
              MOVWF     BEXP
              MOVF      BARGB2,W       ;keep for postnormalization
              MOVWF		BARGB3
              MOVF      BARGB1,W
              MOVWF     BARGB2
              MOVF      BARGB0,W
              MOVWF     BARGB1
              CLRF      BARGB0

              MOVLW     8
              SUBWF     BEXP,W
              BTFSS     _C             ;if BEXP >= 8, do byte shift
              GOTO      ALIGNB32S
              MOVWF     BEXP
              MOVF      BARGB2,W       ;keep for postnormalization
              MOVWF     BARGB3
              MOVF      BARGB1,W
              MOVWF     BARGB2
              CLRF      BARGB1

              MOVLW     8
              SUBWF     BEXP,W
              BTFSS     _C             ;if BEXP >= 8, BARG = 0 relative to AARG
              GOTO      ALIGNB32S
              MOVF      SIGN,W
              MOVWF     AARGB0
              RETLW     0x00

ALIGNB32S     MOVF      BEXP,W         ;already aligned if BEXP = 0
              BTFSC     _Z
              GOTO      ALIGNED32S

ALOOPB32S     BCF       _C             ;right shift by BEXP
              RRF       BARGB0,F
              RRF       BARGB1,F
              RRF       BARGB2,F
              RRF       BARGB3,F
              DECFSZ    BEXP, F
              GOTO      ALOOPB32S

ALIGNED32S    BTFSS     TEMP,MSB       ;negate if signs opposite
              GOTO      AOK32S
              COMF      BARGB3,F
              COMF      BARGB2,F
              COMF      BARGB1,F
              COMF      BARGB0,F
              INCF      BARGB3,F
              BTFSC     _Z
              INCF      BARGB2,F
              BTFSC     _Z
              INCF      BARGB1,F
              BTFSC     _Z
              INCF      BARGB0,F

AOK32S        MOVF      BARGB3,W
              ADDWF   	AARGB3,F
              MOVF      BARGB2,W
              BTFSC     _C
              INCFSZ    BARGB2,W
              ADDWF     AARGB2,F
              MOVF      BARGB1,W
              BTFSC     _C
              INCFSZ    BARGB1,W
              ADDWF     AARGB1,F
              MOVF      BARGB0,W
              BTFSC     _C
              INCFSZ    BARGB0,W
              ADDWF     AARGB0,F

              BTFSC     TEMP,MSB
              GOTO      ACOMP32S
              BTFSS     _C
              GOTO      NRMRND4032S

              RRF       AARGB0,F       ;shift right and increment EXP
              RRF       AARGB1,F
              RRF       AARGB2,F
              RRF       AARGB3,F
              INCFSZ    AEXP,F
              GOTO      NRMRND4032S
              GOTO      SETFOV32S

ACOMP32S      BTFSC     _C
              GOTO      NRM4032S       ;normalize and fix sign
              COMF      AARGB3,F
              COMF      AARGB2,F       ;negate, toggle sign bit and
              COMF      AARGB1,F       ;then normalize
              COMF      AARGB0,F
              INCF      AARGB3,F
              BTFSC     _Z
              INCF      AARGB2,F
              BTFSC     _Z
              INCF      AARGB1,F
              BTFSC     _Z
              INCF      AARGB0,F
              MOVLW     0x80
              XORWF     SIGN,F
              GOTO      NRM32S

;
NRMRND4032S   BTFSC     FPFLAGS,RND
              BTFSS     AARGB2,LSB
              GOTO      FIXSIGN32S
              BTFSS     AARGB3,MSB     ;round if next bit is set
              GOTO      FIXSIGN32S
              INCF      AARGB2,F
              BTFSC     _Z
              INCF      AARGB1,F
              BTFSC     _Z
              INCF      AARGB0,F

              BTFSS     _Z             ;has rounding caused carryout?
              GOTO      FIXSIGN32S
              RRF       AARGB0,F       ;if so, right shift
              RRF       AARGB1,F
              RRF       AARGB2,F
              INCF      EXP,F
              BTFSC     _Z             ;check for overflow
              GOTO      SETFOV32S
              GOTO      FIXSIGN32S

NRM32S         CLRF      TEMP           ;clear exponent decrement
              MOVF      AARGB0,W       ;test if highbyte=0
              BTFSS     _Z
              GOTO      NORM3232S1
              MOVF      AARGB1,W       ;if so, shift 8 bits by move
              MOVWF     AARGB0
              MOVF      AARGB2,W
              MOVWF     AARGB1
              CLRF      AARGB2
              BSF       TEMP,3         ;increase decrement by 8

              MOVF      AARGB0,W       ;test if highbyte=0
              BTFSS     _Z
              GOTO      NORM3232S1
              MOVF      AARGB1,W       ;if so, shift 8 bits by move
              MOVWF     AARGB0
              CLRF      AARGB1
              BCF       TEMP,3         ;increase decrement by 8
              BSF       TEMP,4
        
              MOVF      AARGB0,W       ;if highbyte=0, result=0
              BTFSC     _Z
              GOTO      RES032S

NORM3232S1    MOVF      TEMP,W
              SUBWF     EXP,F
              BTFSS     _Z
              BTFSS     _C
              GOTO      SETFUN32S
              BCF       _C             ;clear carry bit

NORM3232S2    BTFSC     AARGB0,MSB     ;if MSB=1, normalization done
              GOTO      FIXSIGN32S
              RLF       AARGB2,F       ;otherwise, shift left and 
              RLF       AARGB1,F       ;decrement EXP
              RLF       AARGB0,F
              DECFSZ    EXP,F
              GOTO      NORM3232S2
              GOTO      SETFUN32S     ;underflow if EXP=0

FIXSIGN32S    BTFSS     SIGN,MSB
              BCF       AARGB0,MSB     ;clear explicit MSB if positive
              RETLW     0x00

SETFOV32S     BSF       FPFLAGS,FOV    ;set floating point underflag
              BTFSS     FPFLAGS,SAT    ;test for saturation
              RETLW     0xFF           ;return error code in WREG

              MOVLW     0xFF
              MOVWF     AEXP           ;saturate to largest floating
              MOVWF     AARGB0         ;point number = 0x FF 7F FF FF
              MOVWF     AARGB1         ;modulo the appropriate sign bit
              MOVWF     AARGB2
              RLF       SIGN,F
              RRF       AARGB0,F
              RETLW     0xFF           ;return error code in WREG


SETFUN32S     BSF       FPFLAGS,FUN    ;set floating point underflag
              BTFSS     FPFLAGS,SAT    ;test for saturation
              RETLW     0xFF           ;return error code in WREG

              MOVLW     0x01           ;saturate to smallest floating
              MOVWF     AEXP           ;point number = 0x 01 00 00 00
              CLRF      AARGB0         ;modulo the appropriate sign bit
              CLRF      AARGB1
              CLRF      AARGB2
              RLF       SIGN,F
              RRF       AARGB0,F
              RETLW     0xFF           ;return error code in WREG

RES032S       CLRF      AARGB0         ;result equals zero
              CLRF      AARGB1         ;clear variables
              CLRF      AARGB2
              CLRF      AARGB3
              CLRF      EXP
              RETLW     0x00

NRM4032S      CLRF      TEMP           ;clear exponent decrement
              MOVF      AARGB0,W       ;test if highbyte=0
              BTFSS     _Z
              GOTO      NORM4032S1
              MOVF      AARGB1,W       ;if so, shift 8 bits by move
              MOVWF     AARGB0
              MOVF      AARGB2,W
              MOVWF     AARGB1
              MOVF      AARGB3,W
              MOVWF     AARGB2
              CLRF      AARGB3
              BSF       TEMP,3         ;increase decrement by 8

              MOVF      AARGB0,W       ;test if highbyte=0
              BTFSS     _Z
              GOTO      NORM4032S1
              MOVF      AARGB1,W       ;if so, shift 8 bits by move
              MOVWF     AARGB0
              MOVF      AARGB2,W
              MOVWF     AARGB1
              CLRF      AARGB2
              BCF       TEMP,3         ;increase decrement by 8
              BSF       TEMP,4
        
              MOVF      AARGB0,W       ;test if highbyte=0
              BTFSS     _Z
              GOTO      NORM4032S1
              MOVF      AARGB1,W       ;if so, shift 8 bits by move
              MOVWF     AARGB0
              CLRF      AARGB1
              BSF       TEMP,3         ;increase decrement by 8
        
              MOVF      AARGB0,W       ;if highbyte=0, result=0
              BTFSC     _Z
              GOTO      RES032S

NORM4032S1    MOVF      TEMP,W
              SUBWF     EXP,F
              BTFSS     _Z
              BTFSS     _C
              GOTO      SETFUN32S
              BCF       _C             ;clear carry bit

NORM4032S2    BTFSC     AARGB0,MSB     ;if MSB=1, normalization done
              GOTO      NRMRND4032S
              RLF       AARGB3,F       ;otherwise, shift left and 
              RLF       AARGB2,F       ;decrement EXP
              RLF       AARGB1,F
              RLF       AARGB0,F
              DECFSZ    EXP, F
              GOTO      NORM4032S2
              GOTO      SETFUN32S 

#endasm
}
