#pragma option t        32-bit floating point to 32-bit integer conversion


void int3232(void)
{
#asm
;       PIC16 32 BIT FLOATING POINT LIBRARY     VERSION 2.8  1996/10/07
;
;       Binary operations: input in AEXP,AARG and BEXP,BARG with output in AEXP,AARG
;
;       All routines return WREG = 0x00 for successful completion, and WREG = 0xFF
;       for an error condition specified in FPFLAGS.
;
;*****************************************************************************************
;*****************************************************************************************
;
;       Float to integer conversion
;
;       Input:  32 bit floating point number in AEXP, AARGB0, AARGB1, AARGB2
;
;       Output: 32 bit 2's complement integer right justified in AARGB0, AARGB1, AARGB2,
;               AARGB3
;
;       Result: AARG  <--  INT( AARG )
;
;       PM: 125                                              DM: 7
;
;*****************************************************************************************
;
INT3232       CLRF      AARGB3
              MOVF      EXP,W          ;test for zero argument
              BTFSC     _Z
              RETLW     0x00

              MOVF      AARGB0,W       ;save sign in SIGN
              MOVWF     SIGN
              BSF       AARGB0,MSB     ;make MSB explicit

              MOVLW     0x1F+EXPBIAS   ;remove bias from EXP
              SUBWF     EXP,F
              BTFSS     EXP,MSB        ;if < 0, result=0
              GOTO      SETIOV32
              COMF      EXP,F
              INCF      EXP,F      

              MOVLW     8              ;do byte shift if EXP >= 8
              SUBWF     EXP,W
              BTFSS     _C
              GOTO      TSHIFT3232
              MOVWF     EXP
              RLF       AARGB3,F       ;rotate next bit for rounding
              MOVF      AARGB2,W
              MOVWF     AARGB3
              MOVF      AARGB1,W
              MOVWF     AARGB2
              MOVF      AARGB0,W
              MOVWF     AARGB1
              CLRF      AARGB0

              MOVLW     8              ;do another byte shift if EXP >= 8
              SUBWF     EXP,W
              BTFSS     _C
              GOTO      TSHIFT3232
              MOVWF     EXP
              RLF       AARGB3,F       ;rotate next bit for rounding
              MOVF      AARGB2,W
              MOVWF     AARGB3
              MOVF      AARGB1,W
              MOVWF     AARGB2
              CLRF      AARGB1

              MOVLW     8              ;do another byte shift if EXP >= 8
              SUBWF     EXP,W
              BTFSS     _C
              GOTO      TSHIFT3232
              MOVWF     EXP
              RLF       AARGB3,F       ;rotate next bit for rounding
              MOVF      AARGB2,W
              MOVWF     AARGB3
              CLRF      AARGB2

              MOVLW     8              ;do another byte shift if EXP >= 8
              SUBWF     EXP,W
              BTFSS     _C
              GOTO      TSHIFT3232
              MOVWF     EXP
              RLF       AARGB3,F       ;rotate next bit for rounding
              CLRF      AARGB3
              MOVF      EXP,W
              BTFSS     _Z
              BCF       _C
              GOTO      SHIFT3232OK

TSHIFT3232    MOVF      EXP,W          ;shift completed if EXP = 0
              BTFSC     _Z
              GOTO      SHIFT3232OK

SHIFT3232     BCF      _C
              RRF      AARGB0,F        ;right shift by EXP
              RRF      AARGB1,F
              RRF      AARGB2,F
              RRF      AARGB3,F
              DECFSZ   EXP,F
              GOTO     SHIFT3232

SHIFT3232OK   BTFSC    FPFLAGS,RND
              BTFSS    ACCB3,LSB
              GOTO     INT3232OK
              BTFSS    _C
              GOTO     INT3232OK
              INCF     AARGB3,F
              BTFSC    _Z
              INCF     AARGB2,F
              BTFSC    _Z
              INCF     AARGB1,F
              BTFSC    _Z
              INCF     AARGB0,F
              BTFSC    AARGB0,MSB      ;test for overflow
              GOTO     SETIOV3224I

INT3232OK     BTFSS    SIGN,MSB        ;if sign bit set, negate               
              RETLW    0x00
              COMF     AARGB0,F
              COMF     AARGB1,F
              COMF     AARGB2,F
              COMF     AARGB3,F
              INCF     AARGB3,F
              BTFSC    _Z
              INCF     AARGB2,F
              BTFSC    _Z
              INCF     AARGB1,F
              BTFSC    _Z
              INCF     AARGB0,F
              RETLW    0x00

IRES032       CLRF     AARGB0          ;integer result equals zero
              CLRF     AARGB1
              CLRF     AARGB2
              CLRF     AARGB3
              RETLW    0x00

SETIOV32      BSF      FPFLAGS,IOV     ;set integer overflow flag
              BTFSS    FPFLAGS,SAT     ;test for saturation
              RETLW    0xFF            ;return error code in WREG

              CLRF     AARGB0          ;saturate to largest two's
              BTFSS    SIGN,MSB        ;complement 32 bit integer
              MOVLW    0xFF
              MOVWF    AARGB0          ;SIGN = 0, 0x 7F FF FF FF
              MOVWF    AARGB1          ;SIGN = 1, 0x 80 00 00 00
              MOVWF    AARGB2
              MOVWF    AARGB3
              RLF      SIGN,F
              RRF      AARGB0,F
              RETLW    0xFF            ;return error code in WREG

SETIOV3224I   BSF      FPFLAGS,IOV     ;set integer overflow flag
              BTFSS    FPFLAGS,SAT     ;test for saturation
              RETLW    0xFF            ;return error code in WREG

              CLRF     AARGB0          ;saturate to largest two's
              BTFSS    SIGN,MSB        ;complement 24 bit integer
              MOVLW    0xFF
              MOVWF    AARGB0          ;SIGN = 0, 0x 7F FF FF
              MOVWF    AARGB1          ;SIGN = 1, 0x 80 00 00
              MOVWF    AARGB2
              RLF      SIGN,F
              RRF      AARGB0,F
              RETLW    0xFF            ;return error code in WREG

#endasm
}
