/*       MATH16C INCLUDE FILE      VERSION 2.3 - 1996/08/06  Modified for C

    IMPORTANT NOTE: The math library variables herein are the same variables referenced in the 
    MATH16.inc assembly langauge file. This format of these variables has been modified so as 
    to use them with MPLAB-C. All variable memory allocation will now be handled by the compiler.

	This file replaces the DEV_FAM.inc and MATH16.inc files. The purpose of the DEV_FAM.inc file 
    is to evaluate the LIST directive in the source code and then set the appropriate labels to 
    TRUE (these labels indicate which type of processor is to be used and therefore RAM 
    allocation is based on the device used). When the assembler was invoked it evaluates 
    the IF directives in the MATH16.inc file. This results in math variable assignment to RAM 
    based on the processor type used. Since the C-compiler does all variable assignment, there 
    is no need for these files files. 

    	
    GENERAL MATH LIBRARY DEFINITIONS

	general literal constants
*/

#define  B0     0
#define  B1     1
#define  B2     2
#define  B3     3
#define  B4     4
#define  B5     5
#define  B6     6
#define  B7     7

#define MSB     7
#define LSB     0


//     define commonly used bits

//     STATUS bit definitions
#define _C     STATUS,0
#define _Z     STATUS,2


//     general register variables used in fixed point math routines

unsigned int ACCB0;            //MSB
unsigned int ACC @ ACCB0;
unsigned int ACCB1;    
unsigned int ACCB2; 
unsigned int ACCB3;    
unsigned int ACCB4;
unsigned int ACCB5; 
unsigned int ACCB6;     
unsigned int ACCB7;            //LSB

unsigned int SIGN;             //save location for sign in MSB

unsigned int TEMPB0;
unsigned int TEMP @ TEMPB0;    //temporary storage           
unsigned int TEMPB1;          
unsigned int TEMPB2;          
unsigned int TEMPB3;           


//     binary operation arguments used in floating point math routines

unsigned int AARGB0 @ ACCB0;   //most significant byte of argument A   
unsigned int AARGB1 @ ACCB1;          
unsigned int AARGB2 @ ACCB2;          
unsigned int AARGB3 @ ACCB3;           
unsigned int AARGB4 @ ACCB4;           
unsigned int AARGB5 @ ACCB5;           
unsigned int AARGB6 @ ACCB6;           
unsigned int AARGB7 @ ACCB7;   //least significant byte of argument A     
unsigned int AARG @ ACC;       //most significant byte of argument A

unsigned int BARGB0;
unsigned int BARG @ BARGB0;    //most significant byte of argument B           
unsigned int BARGB1;          
unsigned int BARGB2;          
unsigned int BARGB3;          


/*
       Note that AARG and ACC varibales reference the same storage locations as
       do other variables.

*********************************************************************************************
*********************************************************************************************
        FIXED POINT SPECIFIC DEFINITIONS

        remainder storage
*/

unsigned int REMB0 @ ACCB4;           
unsigned int REMB1 @ ACCB5;           
unsigned int REMB2 @ ACCB6;            
unsigned int REMB3 @ ACCB7;    //most significant byte of remainder

unsigned int LOOPCOUNT;        //general purpose loop counter 

/*
*********************************************************************************************
*********************************************************************************************
        FLOATING POINT SPECIFIC DEFINITIONS

        literal constants
*/

#define EXPBIAS     0x7F

//       biased exponents

unsigned int EXP;             //8 bit biased exponent
unsigned int AEXP @ EXP;      //8 bit biased exponent for argument A
unsigned int BEXP;            //8 bit biased exponent for argument B


//       floating point library exception flags

unsigned int FPFLAGS;        //floating point library exception flags
                
#define IOV     0            //bit0 = integer overflow flag

#define FOV     1            //bit1 = floating point overflow flag

#define FUN     2            //bit2 = floating point underflow flag

#define FDZ     3            //bit3 = floating point divide by zero flag

#define NAN     4            //bit4 = not-a-number exception flag

#define DOM     5            //bit5 = domain error exception flag
       
#define RND     6            //bit6 = floating point rounding flag, 0 = truncation
                             //1 = unbiased rounding to nearest LSB

#define SAT     7            //bit7 = floating point saturate flag, 0 = terminate on
                             //exception without saturation, 1 = terminate on
                             //exception with saturation to appropriate value



 #ifdef __PIC14              

 // ELEMENTARY FUNCTION MEMORY
 // for 14bit core only

 unsigned int  CEXP;
 unsigned int  CARGB0;
 unsigned int  CARGB1;
 unsigned int  CARGB2;
 unsigned int  CARGB3;

 unsigned int  DEXP;
 unsigned int  DARGB0;
 unsigned int  DARGB1;
 unsigned int  DARGB2;
 unsigned int  DARGB3;

 unsigned int  EEXP;
 unsigned int  EARGB0;
 unsigned int  EARGB1;
 unsigned int  EARGB2;
 unsigned int  EARGB3;

 #endif


//	24 BIT FLOATING POINT CONSTANTS
//	Machine precision
#define	MACHEP24EXP    0x6F     // 1.52587890625e-5 = 2**-16
#define MACHEP24B0     0x00
#define MACHEP24B1     0x00


//  Maximum argument to EXP24
#define MAXLOG24EXP    0x85     // 88.7228391117 = log(2**128)
#define MAXLOG24B0     0x31
#define MAXLOG24B1     0x72


//  Minimum argument to EXP24
#define MINLOG24EXP    0x85     // -87.3365447506 = log(2**-126)
#define MINLOG24B0     0xAE
#define MINLOG24B1     0xAC


//  Maximum argument to EXP1024
#define MAXLOG1024EXP  0x84     // 38.531839445 = log10(2**128)
#define MAXLOG1024B0   0x1A
#define MAXLOG1024B1   0x21


//  Minimum argument to EXP1024
#define MINLOG1024EXP  0x84     // -37.9297794537 = log10(2**-126)
#define MINLOG1024B0   0x97
#define MINLOG1024B1   0xB8


//  Maximum representable number before overflow
#define MAXNUM24EXP    0xFF     // 6.80554349248E38 = (2**128) * (2 - 2**-15)
#define MAXNUM24B0     0x7F
#define MAXNUM24B1     0xFF


//  Minimum representable number before underflow
#define MINNUM24EXP    0x01     // 1.17549435082E-38 = (2**-126) * 1
#define MINNUM24B0     0x00
#define MINNUM24B1     0x00


//  Loss threshhold for argument to SIN24 and COS24
#define LOSSTHR24EXP   0x8B     // 4096 = sqrt(2**24)
#define LOSSTHR24B0    0x00
#define LOSSTHR24B1    0x00



//	32 BIT FLOATING POINT CONSTANTS

//  Machine precision
#define MACHEP32EXP    0x67     // 5.96046447754E-8 = 2**-24
#define MACHEP32B0     0x00
#define MACHEP32B1     0x00
#define MACHEP32B2     0x00


// Maximum argument to EXP32
#define MAXLOG32EXP    0x85     // 88.7228391117 = log(2**128)
#define MAXLOG32B0     0x31
#define MAXLOG32B1     0x72
#define MAXLOG32B2     0x18


//  Minimum argument to EXP32
#define MINLOG32EXP    0x85     // -87.3365447506 = log(2**-126)
#define MINLOG32B0     0xAE
#define MINLOG32B1     0xAC
#define MINLOG32B2     0x50


//  Maximum argument to EXP1032
#define MAXLOG1032EXP  0x84     // 38.531839445 = log10(2**128)
#define MAXLOG1032B0   0x1A
#define MAXLOG1032B1   0x20
#define MAXLOG1032B2   0x9B


//  Minimum argument to EXP1032
#define MINLOG1032EXP  0x84     // -37.9297794537 = log10(2**-126)
#define MINLOG1032B0   0x97
#define MINLOG1032B1   0xB8
#define MINLOG1032B2   0x18


//  Maximum representable number before overflow
#define MAXNUM32EXP    0xFF     // 6.80564774407E38 = (2**128) * (2 - 2**-23)
#define MAXNUM32B0     0x7F
#define MAXNUM32B1     0xFF
#define MAXNUM32B2     0xFF


//  Minimum representable number before underflow
#define MINNUM32EXP    0x01     // 1.17549435082E-38 = (2**-126) * 1
#define MINNUM32B0     0x00
#define MINNUM32B1     0x00
#define MINNUM32B2     0x00


//  Loss threshhold for argument to SIN32 and COS32
#define LOSSTHR32EXP   0x8B     // 4096 = sqrt(2**24)
#define LOSSTHR32B0    0x00
#define LOSSTHR32B1    0x00
#define LOSSTHR32B2    0x00


