#pragma option t        32-bit normalization of unnormalized 32-bit floating point numbers


void nrm3232(void)
{
#asm
;       PIC16 32 BIT FLOATING POINT LIBRARY     VERSION 2.8  1996/10/07
;
;       Binary operations: input in AEXP,AARG and BEXP,BARG with output in AEXP,AARG
;
;       All routines return WREG = 0x00 for successful completion, and WREG = 0xFF
;       for an error condition specified in FPFLAGS.
;
;*****************************************************************************************
;*****************************************************************************************
;
;       Normalization routine
;
;       Input:  32 bit unnormalized floating point number in AEXP, AARGB0, AARGB1,
;               AARGB2, with sign in SIGN,MSB
;
;       Output: 32 bit normalized floating point number in AEXP, AARGB0, AARGB1, AARGB2
;
;       Result: AARG  <--  NORMALIZE( AARG )
;
;       PM: 55                                      DM: 7
;*****************************************************************************************
;
NRM3232       CLRF      TEMP           ;clear exponent decrement
              MOVF      AARGB0,W       ;test if highbyte=0
              BTFSS     _Z
              GOTO      NORM3232N3
              MOVF      AARGB1,W       ;if so, shift 8 bits by move
              MOVWF     AARGB0
              MOVF      AARGB2,W
              MOVWF     AARGB1
              CLRF      AARGB2
              BSF       TEMP,3         ;increase decrement by 8

              MOVF      AARGB0,W       ;test if highbyte=0
              BTFSS     _Z
              GOTO      NORM3232N3
              MOVF      AARGB1,W       ;if so, shift 8 bits by move
              MOVWF     AARGB0
              CLRF      AARGB1
              BCF       TEMP,3         ;increase decrement by 8
              BSF       TEMP,4
        
              MOVF      AARGB0,W       ;if highbyte=0, result=0
              BTFSC     _Z
              GOTO      RES032N3

NORM3232N3    MOVF      TEMP,W
              SUBWF     EXP,F
              BTFSS     _Z
              BTFSS     _C
              GOTO      SETFUN32N3
              BCF       _C             ;clear carry bit

NORM3232AN3   BTFSC     AARGB0,MSB     ;if MSB=1, normalization done
              GOTO      FIXSIGN32N3
              RLF       AARGB2,F       ;otherwise, shift left and 
              RLF       AARGB1,F       ;decrement EXP
              RLF       AARGB0,F
              DECFSZ    EXP,F
              GOTO      NORM3232AN3
              GOTO      SETFUN32N3     ;underflow if EXP=0

FIXSIGN32N3   BTFSS     SIGN,MSB
              BCF       AARGB0,MSB     ;clear explicit MSB if positive
              RETLW     0x00

RES032N3      CLRF      AARGB0         ;result equals zero
              CLRF      AARGB1 
              CLRF      AARGB2
              CLRF   	AARGB3
              CLRF      EXP
              RETLW     0x00

SETFUN32N3    BSF       FPFLAGS,FUN    ;set floating point underflag
              BTFSS     FPFLAGS,SAT    ;test for saturation
              RETLW     0xFF           ;return error code in WREG

              MOVLW     0x01           ;saturate to smallest floating
              MOVWF     AEXP           ;point number = 0x 01 00 00 00
              CLRF      AARGB0         ;modulo the appropriate sign bit
              CLRF      AARGB1
              CLRF      AARGB2
              RLF       SIGN,F
              RRF       AARGB0,F
              RETLW     0xFF           ;return error code in WREG

#endasm
}
