#pragma option t        32-bit normalization of unnormalized 40-bit floating point numbers


void nrm4032(void)
{
#asm
;       PIC16 32 BIT FLOATING POINT LIBRARY     VERSION 2.8  1996/10/07
;
;       Binary operations: input in AEXP,AARG and BEXP,BARG with output in AEXP,AARG
;
;       All routines return WREG = 0x00 for successful completion, and WREG = 0xFF
;       for an error condition specified in FPFLAGS.
;
;*****************************************************************************************
;*****************************************************************************************
;       Normalization routine
;
;       Input:  40 bit unnormalized floating point number in AEXP, AARGB0, AARGB1,
;               AARGB2, AARGB3 with sign in SIGN,MSB
;
;       Output: 32 bit normalized floating point number in AEXP, AARGB0, AARGB1, AARGB2,
;               AARGB3
;
;       Result: AARG  <--  NORMALIZE( AARG )
;
;       PM: 97                                      DM: 8
;****************************************************************************************
;
NRM4032       CLRF      TEMP           ;clear exponent decrement
              MOVF      AARGB0,W       ;test if highbyte=0
              BTFSS     _Z
              GOTO      NORM4032N4
              MOVF      AARGB1,W       ;if so, shift 8 bits by move
              MOVWF     AARGB0
              MOVF      AARGB2,W
              MOVWF     AARGB1
              MOVF      AARGB3,W
              MOVWF     AARGB2
              CLRF      AARGB3
              BSF       TEMP,3         ;increase decrement by 8

              MOVF      AARGB0,W       ;test if highbyte=0
              BTFSS     _Z
              GOTO      NORM4032N4
              MOVF      AARGB1,W       ;if so, shift 8 bits by move
              MOVWF     AARGB0
              MOVF      AARGB2,W
              MOVWF     AARGB1
              CLRF      AARGB2
              BCF       TEMP,3         ;increase decrement by 8
              BSF       TEMP,4
        
              MOVF      AARGB0,W       ;test if highbyte=0
              BTFSS     _Z
              GOTO      NORM4032N4
              MOVF      AARGB1,W       ;if so, shift 8 bits by move
              MOVWF     AARGB0
              CLRF      AARGB1
              BSF       TEMP,3         ;increase decrement by 8
        
              MOVF      AARGB0,W       ;if highbyte=0, result=0
              BTFSC     _Z
              GOTO      RES032N4

NORM4032N4    MOVF      TEMP,W
              SUBWF     EXP,F
              BTFSS     _Z
              BTFSS     _C
              GOTO      SETFUN32N4
              BCF       _C             ;clear carry bit

NORM4032AN4   BTFSC     AARGB0,MSB     ;if MSB=1, normalization done
              GOTO      NRMRND4032N4
              RLF       AARGB3,F       ;otherwise, shift left and 
              RLF       AARGB2,F       ;decrement EXP
              RLF       AARGB1,F
              RLF       AARGB0,F
              DECFSZ    EXP, F
              GOTO      NORM4032AN4
              GOTO      SETFUN32N4     ;underflow if EXP=0

NRMRND4032N4  BTFSC     FPFLAGS,RND
              BTFSS     AARGB2,LSB
              GOTO      FIXSIGN32N4
              BTFSS     AARGB3,MSB     ;round if next bit is set
              GOTO      FIXSIGN32N4
              INCF      AARGB2,F
              BTFSC     _Z
              INCF      AARGB1,F
              BTFSC     _Z
              INCF      AARGB0,F

              BTFSS     _Z             ;has rounding caused carryout?
              GOTO      FIXSIGN32N4
              RRF       AARGB0,F       ;if so, right shift
              RRF       AARGB1,F
              RRF       AARGB2,F
              INCF      EXP, F
              BTFSC     _Z             ;check for overflow
              GOTO      SETFOV32N4
              GOTO      FIXSIGN32N4

FIXSIGN32N4   BTFSS     SIGN,MSB
              BCF       AARGB0,MSB     ;clear explicit MSB if positive
              RETLW     0x00

RES032N4      CLRF      AARGB0         ;result equals zero
              CLRF      AARGB1 
              CLRF      AARGB2
              CLRF      AARGB3
              CLRF      EXP
              RETLW     0x00

SETFUN32N4    BSF       FPFLAGS,FUN    ;set floating point underflag
              BTFSS     FPFLAGS,SAT    ;test for saturation
              RETLW     0xFF           ;return error code in WREG

              MOVLW     0x01           ;saturate to smallest floating
              MOVWF     AEXP           ;point number = 0x 01 00 00 00
              CLRF      AARGB0         ;modulo the appropriate sign bit
              CLRF      AARGB1
              CLRF      AARGB2
              RLF       SIGN, F
              RRF       AARGB0, F
              RETLW     0xFF           ;return error code in WREG

SETFOV32N4    BSF       FPFLAGS,FOV    ;set floating point underflag
              BTFSS     FPFLAGS,SAT    ;test for saturation
              RETLW     0xFF           ;return error code in WREG

              MOVLW     0xFF
              MOVWF     AEXP           ;saturate to largest floating
              MOVWF     AARGB0         ;point number = 0x FF 7F FF FF
              MOVWF     AARGB1         ;modulo the appropriate sign bit
              MOVWF     AARGB2
              RLF       SIGN, F
              RRF       AARGB0, F
              RETLW     0xFF           ;return error code in WREG

#endasm
}
